/*
	Zoom-Rect-er
	DTS Code Snippet
	
	1/6/92	Steve Falkenburg
	
	This snippet shows how to do "Finder" style zooming between two rectangles.
	The boolean flag "kZoomLarger" controls the proportional direction of the zooming.
	
	To get the two rectangles, you drag them out rubberbanded, and the zoom occurs between
	them.  To quit, click the close box.
	
	If you want to do zooms between windows, open up a port with the dimensions of the desktop
	(from GetGrayRgn()).
	
	DON'T use this as a sample of how to do rubberband drawing!!!  It's sort of hacked
	together bypassing the event mechanism and just using Button().
*/

	

#define	kNumSteps		14
#define	kRectsVisible	4
#define	kZoomRatio		.7
#define	kDelayTicks		1

#define	kZoomLarger		true		// change this to zoom "inward"


void InitStuff(void);
void ZoomRect(Boolean zoomOut,Rect *smallRect, Rect *bigRect);
void CalcRect(Rect *theRect,Rect *smallRect,Rect *bigRect,double stepValue);
Boolean GetRects(Rect *zoomFrom,Rect *zoomTo);
void FixRect(Rect *theRect,Rect *rightRect);

Boolean gDone;

void main(void)
{
	WindowPtr window;
	Rect bounds = {44,12,330,500},zoomFrom,zoomTo;
		
	InitStuff();
	window = NewWindow(nil,&bounds,"\pDrag Two Rects to Zoom",true,documentProc,(WindowPtr)-1L,true,0);
	SetPort(window);
	
	do {
		if (GetRects(&zoomFrom,&zoomTo))
			ZoomRect(kZoomLarger,&zoomFrom,&zoomTo);
		EraseRect(&window->portRect);
	}
	while (!gDone);
	
	FlushEvents(everyEvent,0);
}

	
void InitStuff(void)
{
	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();
	FlushEvents(everyEvent,0);
}


void ZoomRect(Boolean zoomLarger,Rect *smallRect, Rect *bigRect)
{
	double firstStep,stepValue,trailer,zoomRatio;
	short i,step;
	Rect curRect;
	unsigned long ticks;
	
	PenPat(qd.gray);
	PenMode(patXor);
	
	
	firstStep=kZoomRatio;
	for (i=0; i<kNumSteps; i++) {
		firstStep *= kZoomRatio;
	}

	if (!zoomLarger) {
		zoomRatio = 1.0/kZoomRatio;
		firstStep = 1.0-firstStep;
	}
	else
		zoomRatio = kZoomRatio;
		
	trailer = firstStep;
	stepValue = firstStep;
	for (step=0; step<(kNumSteps+kRectsVisible); step++) {
	
		// draw new frame
		
		if (step<kNumSteps) {
			stepValue /= zoomRatio;
			CalcRect(&curRect,smallRect,bigRect,stepValue);
			FrameRect(&curRect);
		}
		
		// erase old frame
		
		if (step>=kRectsVisible) {
			trailer /= zoomRatio;
			CalcRect(&curRect,smallRect,bigRect,trailer);
			FrameRect(&curRect);
		}

		Delay(kDelayTicks,&ticks);
	}

	PenNormal();
}


void CalcRect(Rect *theRect,Rect *smallRect,Rect *bigRect,double stepValue)
{
	theRect->left = smallRect->left + (short)((double)(bigRect->left-smallRect->left)*stepValue);
	theRect->top = smallRect->top + (short)((double)(bigRect->top-smallRect->top)*stepValue);
	theRect->right = smallRect->right + (short)((double)(bigRect->right-smallRect->right)*stepValue);
	theRect->bottom = smallRect->bottom + (short)((double)(bigRect->bottom-smallRect->bottom)*stepValue);
}


Boolean GetRects(Rect *zoomFrom,Rect *zoomTo)
{
	short numRects = 0;
	EventRecord ev;
	Rect theRect,drawRect;
	Point firstPt,curPt,oldPt,globalPt;
	KeyMap theKeys;
	WindowPtr window;
	
	PenMode(patXor);
	
	do {
		while (!Button());
		
		GetMouse(&globalPt);
		LocalToGlobal(&globalPt);
		if ((FindWindow(globalPt,&window)==inGoAway) && window==FrontWindow()) {
			gDone = true;
			return false;
		}
		
		GetMouse(&firstPt);
		oldPt = firstPt;
		SetRect(&theRect,firstPt.h,firstPt.v,firstPt.h,firstPt.v);
		while (Button()) {
			GetMouse(&curPt);
			if (!EqualPt(curPt,oldPt)) {
				FixRect(&theRect,&drawRect);
				FrameRect(&drawRect);
				oldPt = curPt;
				theRect.right = curPt.h;
				theRect.bottom = curPt.v;
				FixRect(&theRect,&drawRect);
				FrameRect(&drawRect);
			}
		}
		
		FixRect(&theRect,&drawRect);
		if (numRects==0)
			*zoomFrom = drawRect;
		else
			*zoomTo = drawRect;
			
		numRects++;

	} while (numRects<2);
	
	PenNormal();
}


void FixRect(Rect *theRect,Rect *rightRect)
{
	if (theRect->right > theRect->left) {
		rightRect->right = theRect->right;
		rightRect->left = theRect->left;
	}
	else {
		rightRect->right = theRect->left;
		rightRect->left = theRect->right;
	}
	
	if (theRect->bottom > theRect->top) {
		rightRect->bottom = theRect->bottom;
		rightRect->top = theRect->top;
	}
	else {
		rightRect->bottom = theRect->top;
		rightRect->top = theRect->bottom;
	}
	
}
